@@ -392,6 +392,15 @@ grn_ctx_loader_clear(grn_ctx *ctx)
 
 #define IMPL_SIZE ((sizeof(struct _grn_ctx_impl) + (grn_pagesize - 1)) & ~(grn_pagesize - 1))
 
+#ifdef HAVE_MESSAGE_PACK
+static inline int
+grn_msgpack_buffer_write(void *data, const char *buf, unsigned int len)
+{
+  grn_ctx *ctx = (grn_ctx *)data;
+  return grn_bulk_write(ctx, ctx->impl->outbuf, buf, len);
+}
+#endif
+
 static void
 grn_ctx_impl_init(grn_ctx *ctx)
 {
@@ -477,6 +486,10 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->edge = NULL;
   grn_loader_init(&ctx->impl->loader);
   ctx->impl->plugin_path = NULL;
+
+#ifdef HAVE_MESSAGE_PACK
+  msgpack_packer_init(&ctx->impl->msgpacker, ctx, grn_msgpack_buffer_write);
+#endif
 }
 
 void
@@ -1089,6 +1102,14 @@ get_content_mime_type(grn_ctx *ctx, const char *p, const char *pe)
         ctx->impl->mime_type = "image/jpeg";
       }
       break;
+#ifdef HAVE_MESSAGE_PACK
+    case 'm' :
+      if (p + 7 == pe && !memcmp(p, "msgpack", 7)) {
+        ctx->impl->output_type = GRN_CONTENT_MSGPACK;
+        ctx->impl->mime_type = "application/x-msgpack";
+      }
+      break;
+#endif
     case 'p' :
       if (p + 3 == pe && !memcmp(p, "png", 3)) {
         ctx->impl->output_type = GRN_CONTENT_NONE;
