@@ -98,7 +98,30 @@ substitute_query(grn_ctx *ctx, grn_obj *table, grn_obj *column,
   grn_id id;
   grn_rc rc = GRN_END_OF_DATA;
   if ((id = grn_table_get(ctx, table, (const void *)key, (unsigned)key_size))) {
-    grn_obj_get_value(ctx, column, id, dest);
+    if ((column->header.type == GRN_COLUMN_VAR_SIZE) &&
+        ((column->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_VECTOR)) {
+      unsigned int i, n;
+      grn_obj values;
+      GRN_TEXT_INIT(&values, GRN_OBJ_VECTOR);
+      grn_obj_get_value(ctx, column, id, &values);
+      n = grn_vector_size(ctx, &values);
+      if (n > 1) { GRN_TEXT_PUTC(ctx, dest, '('); }
+      for (i = 0; i < n; i++) {
+        const char *value;
+        unsigned int length;
+        if (i > 0) {
+          GRN_TEXT_PUTS(ctx, dest, " OR ");
+        }
+        if (n > 1) { GRN_TEXT_PUTC(ctx, dest, '('); }
+        length = grn_vector_get_element(ctx, &values, i, &value, NULL, NULL);
+        GRN_TEXT_PUT(ctx, dest, value, length);
+        if (n > 1) { GRN_TEXT_PUTC(ctx, dest, ')'); }
+      }
+      if (n > 1) { GRN_TEXT_PUTC(ctx, dest, ')'); }
+      GRN_OBJ_FIN(ctx, &values);
+    } else {
+      grn_obj_get_value(ctx, column, id, dest);
+    }
     rc = GRN_SUCCESS;
   }
   return rc;
@@ -382,8 +405,21 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
     }
     nhits = res ? grn_table_size(ctx, res) : 0;
     LAP(":", "select(%d)", nhits);
-    GRN_OUTPUT_ARRAY_OPEN("RESULT", -1);
+
     if (res) {
+      uint32_t ngkeys;
+      grn_table_sort_key *gkeys = NULL;
+      int result_size = 1;
+      if (!ctx->rc && drilldown_len) {
+        gkeys = grn_table_sort_key_from_str(ctx,
+                                            drilldown, drilldown_len,
+                                            res, &ngkeys);
+        if (gkeys) {
+          result_size += ngkeys;
+        }
+      }
+      GRN_OUTPUT_ARRAY_OPEN("RESULT", result_size);
+
       if (scorer && scorer_len) {
         grn_obj *v;
         GRN_EXPR_CREATE_FOR_QUERY(ctx, res, scorer_, v);
@@ -435,11 +471,9 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
       }
       LAP(":", "output(%d)", limit);
       if (!ctx->rc && drilldown_len) {
-        uint32_t i, ngkeys;
-        grn_table_sort_key *gkeys;
+        uint32_t i;
         grn_table_group_result g = {NULL, 0, 0, 1, GRN_TABLE_GROUP_CALC_COUNT, 0};
-        if ((gkeys = grn_table_sort_key_from_str(ctx, drilldown,
-                                                 drilldown_len, res, &ngkeys))) {
+        if (gkeys) {
           for (i = 0; i < ngkeys; i++) {
             if ((g.table = grn_table_create_for_group(ctx, NULL, 0, NULL,
                                                       GRN_TABLE_HASH_KEY|GRN_OBJ_WITH_SUBREC,
@@ -494,6 +528,8 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
         }
       }
       if (res != table_) { grn_obj_unlink(ctx, res); }
+    } else {
+      GRN_OUTPUT_ARRAY_OPEN("RESULT", 0);
     }
     GRN_OUTPUT_ARRAY_CLOSE();
     if (!ctx->rc && cacheable && cache_key_size <= GRN_TABLE_MAX_KEY_SIZE
@@ -799,7 +835,8 @@ proc_table_create(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_d
                              GRN_TEXT_LEN(VAR(2)));
       if (!key_type) {
         ERR(GRN_INVALID_ARGUMENT,
-            "key type doesn't exist: <%.*s>",
+            "[table][create] key type doesn't exist: <%.*s> (%.*s)",
+            GRN_TEXT_LEN(VAR(0)), GRN_TEXT_VALUE(VAR(0)),
             GRN_TEXT_LEN(VAR(2)), GRN_TEXT_VALUE(VAR(2)));
         return NULL;
       }
@@ -809,7 +846,8 @@ proc_table_create(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_d
                                GRN_TEXT_LEN(VAR(3)));
       if (!value_type) {
         ERR(GRN_INVALID_ARGUMENT,
-            "value type doesn't exist: <%.*s>",
+            "[table][create] value type doesn't exist: <%.*s> (%.*s)",
+            GRN_TEXT_LEN(VAR(0)), GRN_TEXT_VALUE(VAR(0)),
             GRN_TEXT_LEN(VAR(3)), GRN_TEXT_VALUE(VAR(3)));
         return NULL;
       }
@@ -829,7 +867,8 @@ proc_table_create(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_d
       grn_obj_unlink(ctx, table);
     }
   } else {
-    ERR(GRN_INVALID_ARGUMENT, "should not create anonymous table");
+    ERR(GRN_INVALID_ARGUMENT,
+        "[table][create] should not create anonymous table");
   }
 exit:
   GRN_OUTPUT_BOOL(!ctx->rc);
@@ -1049,9 +1088,23 @@ proc_column_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_da
                            GRN_TEXT_LEN(VAR(0))))) {
     grn_hash *cols;
     grn_obj *col;
+    int column_list_size = -1;
+#ifdef HAVE_MESSAGE_PACK
+    column_list_size = 1; /* [header, (key), (COLUMNS)] */
+    if ((col = grn_obj_column(ctx, table, KEY_NAME, sizeof(KEY_NAME)-1))) {
+      column_list_size++;
+      grn_obj_unlink(ctx, col);
+    }
     if ((cols = grn_hash_create(ctx, NULL, sizeof(grn_id), 0,
                                 GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY))) {
-      GRN_OUTPUT_ARRAY_OPEN("COLUMN_LIST", -1);
+      column_list_size += grn_table_columns(ctx, table, NULL, 0,
+                                            (grn_obj *)cols);
+      grn_hash_close(ctx, cols);
+    }
+#endif
+    if ((cols = grn_hash_create(ctx, NULL, sizeof(grn_id), 0,
+                                GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY))) {
+      GRN_OUTPUT_ARRAY_OPEN("COLUMN_LIST", column_list_size);
       GRN_OUTPUT_ARRAY_OPEN("HEADER", 8);
       GRN_OUTPUT_ARRAY_OPEN("PROPERTY", 2);
       GRN_OUTPUT_CSTR("id");
@@ -1131,22 +1184,28 @@ proc_column_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_da
   return NULL;
 }
 
+static grn_bool
+is_table(grn_obj *obj)
+{
+  switch (obj->header.type) {
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
+  case GRN_TABLE_NO_KEY:
+  case GRN_TABLE_VIEW:
+    return GRN_TRUE;
+  default:
+    return GRN_FALSE;
+  }
+}
+
 static int
 print_tableinfo(grn_ctx *ctx, grn_obj *table)
 {
   grn_id id;
   grn_obj o;
   const char *path;
-  switch (table->header.type) {
-  case GRN_TABLE_HASH_KEY:
-  case GRN_TABLE_PAT_KEY:
-  case GRN_TABLE_DAT_KEY:
-  case GRN_TABLE_NO_KEY:
-  case GRN_TABLE_VIEW:
-    break;
-  default:
-    return 0;
-  }
+
   id = grn_obj_id(ctx, table);
   path = grn_obj_path(ctx, table);
   GRN_TEXT_INIT(&o, 0);
@@ -1170,9 +1229,29 @@ static grn_obj *
 proc_table_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
   grn_table_cursor *cur;
+  int table_list_length = -1;
+
+#ifdef HAVE_MESSAGE_PACK
+  if (ctx->impl->output_type == GRN_CONTENT_MSGPACK) {
+    table_list_length = 1; /* header */
+    if ((cur = grn_table_cursor_open(ctx, ctx->impl->db, NULL, 0, NULL, 0, 0, -1, 0))) {
+      grn_id id;
+      while ((id = grn_table_cursor_next(ctx, cur)) != GRN_ID_NIL) {
+        grn_obj *o;
+        if ((o = grn_ctx_at(ctx, id))) {
+          if (is_table(o)) {
+            table_list_length++;
+          }
+          grn_obj_unlink(ctx, o);
+        }
+      }
+    }
+  }
+#endif
+
   if ((cur = grn_table_cursor_open(ctx, ctx->impl->db, NULL, 0, NULL, 0, 0, -1, 0))) {
     grn_id id;
-    GRN_OUTPUT_ARRAY_OPEN("TABLE_LIST", -1);
+    GRN_OUTPUT_ARRAY_OPEN("TABLE_LIST", table_list_length);
     GRN_OUTPUT_ARRAY_OPEN("HEADER", 6);
     GRN_OUTPUT_ARRAY_OPEN("PROPERTY", 2);
     GRN_OUTPUT_CSTR("id");
@@ -1202,7 +1281,9 @@ proc_table_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
     while ((id = grn_table_cursor_next(ctx, cur)) != GRN_ID_NIL) {
       grn_obj *o;
       if ((o = grn_ctx_at(ctx, id))) {
-        print_tableinfo(ctx, o);
+        if (is_table(o)) {
+          print_tableinfo(ctx, o);
+        }
         grn_obj_unlink(ctx, o);
       }
     }
@@ -1533,9 +1614,14 @@ proc_delete(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
                                GRN_TEXT_VALUE(VAR(0)),
                                GRN_TEXT_LEN(VAR(0)));
   if (table) {
-    if (GRN_TEXT_LEN(VAR(1)) && GRN_TEXT_LEN(VAR(2))) {
+    if (GRN_TEXT_LEN(VAR(1)) && GRN_TEXT_LEN(VAR(2)) &&
+        !GRN_TEXT_LEN(VAR(3))) {
       ERR(GRN_INVALID_ARGUMENT, "both id and key are specified");
       rc = ctx->rc;
+    } else if (GRN_TEXT_LEN(VAR(3)) &&
+               (GRN_TEXT_LEN(VAR(1)) || GRN_TEXT_LEN(VAR(2)))) {
+      ERR(GRN_INVALID_ARGUMENT, "filter is used with id and/or key");
+      rc = ctx->rc;
     } else if (GRN_TEXT_LEN(VAR(1))) {
       grn_obj *p_key = VAR(1);
       grn_obj key;
@@ -1559,6 +1645,31 @@ proc_delete(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
         ERR(GRN_INVALID_ARGUMENT, "invalid id");
         rc = ctx->rc;
       }
+    } else if (GRN_TEXT_LEN(VAR(3))) {
+      grn_obj *cond, *v;
+      const char *filter = GRN_TEXT_VALUE(VAR(3));
+      unsigned filter_len = GRN_TEXT_LEN(VAR(3));
+
+      GRN_EXPR_CREATE_FOR_QUERY(ctx, table, cond, v);
+      grn_expr_parse(ctx, cond, filter, filter_len,
+                     NULL, GRN_OP_MATCH, GRN_OP_AND,
+                     GRN_EXPR_SYNTAX_SCRIPT);
+      if (!ctx->rc) {
+        grn_obj *records;
+
+        records = grn_table_select(ctx, table, cond, NULL, GRN_OP_OR);
+        if (records) {
+          void *key;
+          GRN_TABLE_EACH(ctx, records, GRN_ID_NIL, GRN_ID_NIL,
+                         result_id, &key, NULL, NULL, {
+            grn_id id = *(grn_id *)key;
+            grn_table_delete_by_id(ctx, table, id);
+          });
+          grn_obj_unlink(ctx, records);
+        }
+      }
+
+      grn_obj_unlink(ctx, cond);
     }
   } else {
     ERR(GRN_INVALID_ARGUMENT, "unknown table name");
@@ -2389,8 +2500,18 @@ func_geo_in_circle(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_
 {
   grn_obj *obj;
   unsigned char r = GRN_FALSE;
-  if (nargs == 3) {
-    r = grn_geo_in_circle(ctx, args[0], args[1], args[2]);
+  grn_geo_approximate_type type = GRN_GEO_APPROXIMATE_RECTANGLE;
+  switch (nargs) {
+  case 4 :
+    if (grn_geo_resolve_approximate_type(ctx, args[3], &type) != GRN_SUCCESS) {
+      break;
+    }
+    /* fallthru */
+  case 3 :
+    r = grn_geo_in_circle(ctx, args[0], args[1], args[2], type);
+    break;
+  default :
+    break;
   }
   if ((obj = GRN_PROC_ALLOC(GRN_DB_UINT32, 0))) {
     GRN_UINT32_SET(ctx, obj, r);
@@ -2416,9 +2537,19 @@ static grn_obj *
 func_geo_distance(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
   grn_obj *obj;
-  double d = 0;
-  if (nargs == 2) {
-    d = grn_geo_distance(ctx, args[0], args[1]);
+  double d = 0.0;
+  grn_geo_approximate_type type = GRN_GEO_APPROXIMATE_RECTANGLE;
+  switch (nargs) {
+  case 3 :
+    if (grn_geo_resolve_approximate_type(ctx, args[2], &type) != GRN_SUCCESS) {
+      break;
+    }
+    /* fallthru */
+  case 2 :
+    d = grn_geo_distance(ctx, args[0], args[1], type);
+    break;
+  default:
+    break;
   }
   if ((obj = GRN_PROC_ALLOC(GRN_DB_FLOAT, 0))) {
     GRN_FLOAT_SET(ctx, obj, d);
@@ -2426,13 +2557,14 @@ func_geo_distance(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_d
   return obj;
 }
 
+/* deprecated. */
 static grn_obj *
 func_geo_distance2(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
   grn_obj *obj;
   double d = 0;
   if (nargs == 2) {
-    d = grn_geo_distance2(ctx, args[0], args[1]);
+    d = grn_geo_distance_sphere(ctx, args[0], args[1]);
   }
   if ((obj = GRN_PROC_ALLOC(GRN_DB_FLOAT, 0))) {
     GRN_FLOAT_SET(ctx, obj, d);
@@ -2440,13 +2572,14 @@ func_geo_distance2(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_
   return obj;
 }
 
+/* deprecated. */
 static grn_obj *
 func_geo_distance3(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
   grn_obj *obj;
   double d = 0;
   if (nargs == 2) {
-    d = grn_geo_distance3(ctx, args[0], args[1]);
+    d = grn_geo_distance_ellipsoid(ctx, args[0], args[1]);
   }
   if ((obj = GRN_PROC_ALLOC(GRN_DB_FLOAT, 0))) {
     GRN_FLOAT_SET(ctx, obj, d);
@@ -2612,7 +2745,8 @@ grn_db_init_builtin_query(grn_ctx *ctx)
   DEF_VAR(vars[0], "table");
   DEF_VAR(vars[1], "key");
   DEF_VAR(vars[2], "id");
-  DEF_COMMAND("delete", proc_delete, 3, vars);
+  DEF_VAR(vars[3], "filter");
+  DEF_COMMAND("delete", proc_delete, 4, vars);
 
   DEF_VAR(vars[0], "max");
   DEF_COMMAND("cache_limit", proc_cache_limit, 1, vars);
@@ -2643,9 +2777,11 @@ grn_db_init_builtin_query(grn_ctx *ctx)
   grn_proc_create(ctx, "geo_distance", 12, GRN_PROC_FUNCTION,
                   func_geo_distance, NULL, NULL, 0, NULL);
 
+  /* deprecated. */
   grn_proc_create(ctx, "geo_distance2", 13, GRN_PROC_FUNCTION,
                   func_geo_distance2, NULL, NULL, 0, NULL);
 
+  /* deprecated. */
   grn_proc_create(ctx, "geo_distance3", 13, GRN_PROC_FUNCTION,
                   func_geo_distance3, NULL, NULL, 0, NULL);
 
