@@ -1527,8 +1527,7 @@ void
 grn_itoh(unsigned int i, char *p, unsigned int len)
 {
   static const char *hex = "0123456789ABCDEF";
-  p += len;
-  *p-- = '\0';
+  p += len - 1;
   while (len--) {
     *p-- = hex[i & 0xf];
     i >>= 4;
