@@ -19,10 +19,43 @@ require 'groonga-test-utils'
 require 'net/http'
 require 'cgi'
 require 'json'
+require 'msgpack'
 
 module GroongaHTTPTestUtils
   include GroongaTestUtils
 
+  module Format
+    module JSON
+      private
+      def output_type
+        nil
+      end
+
+      def content_type
+        "application/json"
+      end
+
+      def parse(response)
+        ::JSON.parse(response)
+      end
+    end
+
+    module MessagePack
+      private
+      def output_type
+        "msgpack"
+      end
+
+      def content_type
+        "application/x-msgpack"
+      end
+
+      def parse(response)
+        ::MessagePack.unpack(response)
+      end
+    end
+  end
+
   def setup_server
     super("http")
   end
@@ -30,7 +63,7 @@ module GroongaHTTPTestUtils
   def teardown_server
     if @groonga_pid
       begin
-        timeout(2) do
+        timeout(5) do
           shutdown_server
           begin
             Process.waitpid(@groonga_pid)
@@ -183,13 +216,13 @@ module GroongaHTTPTestUtils
   end
 
   def load_schedules
-    header = ["month", "day"]
+    header = ["day", "month"]
 
     records = []
     1.upto(12) do |month|
       days = (1..28).to_a.shuffle
       1.upto(10) do
-        records.push([month, days.pop])
+        records.push([days.pop, month])
       end
     end
     records.shuffle!
@@ -204,13 +237,48 @@ module GroongaHTTPTestUtils
   end
 
   def json(object)
-    JSON.generate(object)
+    ::JSON.generate(object)
   end
 
   def success_status_response
     [Result::SUCCESS, 0.0, 0.0]
   end
 
+  def hexdump_hex_part(binary, bytes_per_line)
+    sublines_per_line = 2
+    sublines_delimiter = '  '
+
+    bytes_per_subline = bytes_per_line / sublines_per_line
+    hex_part_length = 2 * bytes_per_line +
+      (bytes_per_line - sublines_per_line) +
+      sublines_delimiter.length * (sublines_per_line - 1)
+
+    hex_sublines = []
+    binary.each_slice(bytes_per_subline) do |binary_subline|
+      hex_sublines << binary_subline.collect {|byte| '%02x' % byte}.join(' ')
+    end
+    hex_sublines.join(sublines_delimiter).ljust(hex_part_length)
+  end
+
+  def hexdump_ascii_part(binary)
+    binary.collect do |byte|
+      (0x20..0x7e).include?(byte) ? byte.chr : '.'
+    end.join
+  end
+
+  def hexdump(str)
+    results = ''
+    bytes_per_line = 16
+    binary_lines = str.bytes.each_slice(bytes_per_line)
+    binary_lines.each do |binary_line|
+      results << hexdump_hex_part(binary_line, bytes_per_line)
+      results << "  "
+      results << hexdump_ascii_part(binary_line)
+      results << "\n"
+    end
+    results
+  end
+
   def assert_response(expected, response, options=nil)
     actual = nil
     options ||= {}
@@ -222,15 +290,20 @@ module GroongaHTTPTestUtils
     case response.content_type
     when "application/json"
       begin
-        actual = JSON.parse(response.body)
-      rescue JSON::ParserError => e
+        actual = ::JSON.parse(response.body)
+      rescue ::JSON::ParserError => e
         raise "JSON ParserError #{e.message}\nJSON is ...\n" \
               "---\n#{response.body}\n---"
       end
-      if actual[0][0].is_a?(Integer)
-        actual[0][1..2] = [0.0, 0.0]
-        actual[0][4] = nil if actual[0][4]
+      normalize_structured_response(actual)
+    when "application/x-msgpack"
+      begin
+        actual = ::MessagePack.unpack(response.body)
+      rescue ::MessagePack::UnpackError => e
+        raise "MessagePack UnpackError #{e.message}\nMessagePack is ...\n" \
+              "---\n#{hexdump(response.body)}---"
       end
+      normalize_structured_response(actual)
     when "text/html"
       actual = utf8(response.body)
     when "text/xml"
@@ -259,6 +332,7 @@ module GroongaHTTPTestUtils
     command_name = options[:command] || :select
     response = get(command_path(command_name, parameters))
     drilldown_results = options[:drilldown_results] || []
+    expected_content_type = options[:content_type] || "application/json"
 
     assert_response([success_status_response,
                      [[[options[:n_hits] || expected.size],
@@ -267,7 +341,7 @@ module GroongaHTTPTestUtils
                       ],
                      *drilldown_results]],
                     response,
-                    :content_type => "application/json",
+                    :content_type => expected_content_type,
                     &block)
   end
 
@@ -285,4 +359,11 @@ module GroongaHTTPTestUtils
       xml
     end
   end
+
+  def normalize_structured_response(response)
+    if response[0][0].is_a?(Integer)
+      response[0][1..2] = [0.0, 0.0]
+      response[0][4] = nil if response[0][4]
+    end
+  end
 end
